#!/usr/local/bin/perl
# Display a form for editing or creating a firewall rule

require './ipfilter-lib.pl';
&ReadParse();
$rules = &get_config();

if ($in{'delsel'}) {
	# Special case - deleting selected rules
	@nums = sort { $b cmp $a } split(/\0/, $in{'d'});
	if (@nums) {
		&lock_file($rule->[$nums[0]]->{'file'});
		foreach $n (@nums) {
			&delete_rule($rules->[$n]);
			}
		&flush_file_lines();
		&unlock_file($rule->[$nums[0]]->{'file'});
		&webmin_log("delsel", undef, undef,
			    { 'count' => scalar(@nums) });
		}
	&redirect("");
	exit;
	}

if ($in{'new'}) {
	&ui_print_header(undef, $text{'edit_title1'}, "");
	$rule = { 'action' => 'pass',
		  'all' => 1,
		  'active' => 1,
		  'dir' => 'in' };
	}
else {
	$rule = $rules->[$in{'idx'}];
	&ui_print_header(undef, $text{'edit_title2'}, "");
	}

print &ui_form_start("save_rule.cgi", "post");
print &ui_hidden("new", $in{'new'}),"\n";
print &ui_hidden("idx", $in{'idx'}),"\n";
print &ui_hidden("before", $in{'before'}),"\n";
print &ui_hidden("after", $in{'after'}),"\n";
@tds = ( "width=20%", undef );

print &ui_table_start($text{'edit_header1'}, "width=100%", 2);

# Comment
print &ui_table_row($text{'edit_cmt'},
		    $rule->{'cmt'} =~ /\n/ ? 
			&ui_textarea("cmt", $rule->{'cmt'}, 3, 50) :
			&ui_textbox("cmt", $rule->{'cmt'}, 50),
		    undef, \@tds);

# Enabled
print &ui_table_row($text{'edit_active'},
		    &ui_radio("active", $rule->{'active'} ? 1 : 0,
			      [ [ 1, $text{'edit_active1'} ],
				[ 0, $text{'edit_active0'} ] ]));

# Rule action and argument
$ra = $rule->{'action'};
push(@action, $ra) if ($ra && &indexof($ra, @actions) < 0);
$acts = "<table cellpadding=1 cellspacing=1>\n";
$i = 0;
foreach $a (@actions) {
	$acts .= "<tr> <td>";
	local $ma = $rule->{'action'} eq $a;
	$acts .= &ui_oneradio("action", $a,
			$text{"laction_".$a} || $text{"action_".$a} || uc($a),
			$ma);
	if ($a eq "block") {
		# Show ICMP block options
		local @codes = ( [ "", $text{'edit_none'} ],
				 [ "rst", $text{'edit_rst'} ] );
		push(@codes, map { [ $_ ] } @icmp_codes);
		push(@codes, [ $rule->{'block-return'} ])
			if ($rule->{'block-return'} &&
			    $rule->{'block-return'} ne "rst" &&
			    &indexof($rule->{'block-return'}, @icmp_codes) < 0);
		$acts .= &ui_select("block_return", $rule->{'block-return'},
				    \@codes, 0, 0, 1)."\n";
		$acts .= &ui_checkbox("block_return_dest", 1,
				      $text{'edit_return_dest'},
				      $rule->{'block-return-dest'});
		}
	elsif ($a eq "log") {
		# Show logging options
		$acts .= &logging_options("log", 1);
		}
	elsif ($a eq "skip") {
		# Show rule to skip to
		$acts .= &ui_textbox("skip", $rule->{'skip'}, 10);
		}
	elsif ($a eq "call") {
		# Show function name
		$acts .= &ui_textbox("call", $rule->{'call'}, 20)."\n".
			 &ui_checkbox("call_now", 1, $text{'edit_callnow'},
				      $rule->{'call-now'});
		}
	$acts .= "</td> </tr>";
	}
$acts .= "</table>\n";
print &ui_table_row($text{'edit_action'}, $acts,
		    undef, \@tds);

print &ui_table_end(),"<br>\n";

# Show section for source and destination
print &ui_table_start($text{'edit_header2'}, "width=100%", 2);

print &ui_table_row($text{'edit_all'},
		    &ui_radio("all", $rule->{'all'} || 0,
			      [ [ 1, $text{'edit_all1'} ],
				[ 0, $text{'edit_all0'} ] ]));

foreach $f ("from", "to") {
	print &ui_table_hr();

	# Address part
	local $ft;
	$ft .= &ui_oneradio($f, "any", $text{'edit_any'},
			    $rule->{$f."-any"} || $rule->{'all'})."\n";
	$ft .= &ui_oneradio($f, "thishost", $text{'edit_thishost'},
			    $rule->{$f."-thishost"})."<br>\n";

	$ft .= &ui_oneradio($f, "host", $text{'edit_host'},
			    $rule->{$f."-host"})."\n";
	$ft .= &ui_textbox($f."_host", $rule->{$f."-host"}, 20)."\n";
	$ft .= "$text{'edit_mask'}\n";
	$ft .= &ui_textbox($f."_mask", $rule->{$f."-mask"}, 13)."<br>\n";

	$ft .= &ui_oneradio($f, "numhost", $text{'edit_host'},
			    $rule->{$f."-numhost"}),"\n";
	$ft .= &ui_textbox($f."_numhost", $rule->{$f."-numhost"}, 20)."\n";
	$ft .= "$text{'edit_nummask'}\n";
	$ft .= &ui_textbox($f."_nummask", $rule->{$f."-nummask"}, 6)."\n".
	       "$text{'edit_opt'}<br>\n";

	print &ui_table_row($text{'edit_'.$f}, $ft);

	# Ports part
	local $pt;

	$pt .= &ui_oneradio($f."_port", "any", $text{'edit_anyport'},
			    !$rule->{$f."-port-comp"} &&
			    !$rule->{$f."-port-range"})."\n";
	$pt .= &ui_oneradio($f."_port", "comp",
	    &text('edit_portcomp',
		  &ui_select($f."_portcomp", $rule->{$f."-port-comp"},
			     [ map { [ $_ ] } @compare_ops ],
			     0, 0, $rule->{$f."-port-comp"} ? 1 : 0),
		  &ui_textbox($f."_portnum", $rule->{$f."-port-num"}, 6)),
	    $rule->{$f."-port-comp"})."<br>\n";
	$pt .= &ui_oneradio($f."_port", "range",
	    &text('edit_portrange',
		  &ui_textbox($f."_portstart", $rule->{$f."-port-start"}, 6),
		  &ui_textbox($f."_portend", $rule->{$f."-port-end"}, 6)),
	    $rule->{$f."-port-range"})."<br>\n";

	print &ui_table_row($text{'edit_port'.$f}, $pt);
	}

print &ui_table_end(),"<br>\n";

# Show section for protocol, ttl, tos
print &ui_table_start($text{'edit_header3'}, "width=100%", 4);

print &ui_table_row($text{'edit_dir'},
    &ui_radio("dir", $rule->{'dir'}, [ [ "in", $text{'dir_in'} ],
				       [ "out", $text{'dir_out'} ] ]));

print &ui_table_row($text{'edit_proto'},
    &ui_select("proto", $rule->{'proto'},
	       [ [ "", $text{'edit_protoany'} ],
		 map { [ $_, uc($_) ] } &list_protocols() ],
	       0, 0, $rule->{'proto'} ? 1 : 0));

print &ui_table_row($text{'edit_tos'},
    &ui_opt_textbox("tos", $rule->{'tos'}, 8, $text{'edit_tosany'}));

print &ui_table_row($text{'edit_ttl'},
    &ui_opt_textbox("ttl", $rule->{'ttl'}, 8, $text{'edit_tosany'}));

print &ui_table_row($text{'edit_on'},
		    &interface_choice("on", $rule->{'on'}));

print &ui_table_row($text{'edit_flags'},
    &ui_opt_textbox("flags1", $rule->{'flags1'}, 8, $text{'edit_flagsany'}).
    " $text{'edit_flags2'} ".&ui_textbox("flags2", $rule->{'flags2'}, 8));

print &ui_table_row($text{'edit_icmp'},
    &ui_select("icmptype", $rule->{'icmp-type'},
	       [ [ "", $text{'edit_icmpany'} ],
		 map { [ $_ ] } @icmp_types ],
	       0, 0, $rule->{'icmp-type'} ? 1 : 0).
    " $text{'edit_icmpcode'} ".
    &ui_select("icmpcode", $rule->{'icmp-type-code'},
	       [ [ "", $text{'edit_codeany'} ],
		 map { [ $_ ] } @icmp_codes ],
	       0, 0, $rule->{'icmp-code'} ? 1 : 0), 3);

print &ui_table_end(),"<br>\n";

# Show section for other options
print &ui_table_start($text{'edit_header4'}, "width=100%", 2);
print "<table>\n";

print "<tr> <td colspan=2>",&ui_checkbox("quick", 1, $text{'edit_quick'},
			       $rule->{'quick'}),"</td> </tr>\n";

print "<tr> <td>",&ui_checkbox("olog", 1, $text{'edit_olog'},
			       $rule->{'olog'}),"</td>\n";
print "<td>",&logging_options("olog", 1),"</td> </tr>\n";

print "<tr> <td>",&ui_checkbox("tag", 1, $text{'edit_tag'},
			       $rule->{'tag'}),"</td>\n";
print "<td>",&ui_textbox("tagid", $rule->{'tag'}, 10),"</td> </tr>\n";

print "<tr> <td>",&ui_checkbox("dup_to", 1, $text{'edit_dupto'},
			       $rule->{'dup-to'}),"</td>\n";
($iface, $ip) = split(/:/, $rule->{'dup-to'});
print "<td>",&interface_choice("dup_toiface", $iface, 1),"\n";
print "$text{'edit_duptoip'}\n";
print &ui_textbox("dup_toip", $ip, 13)," $text{'edit_opt'}</td> </tr>\n";

print "<tr> <td>",&ui_checkbox("fastroute", 1, $text{'edit_fastroute'},
			       $rule->{'fastroute'}),"</td>\n";
print "<td>",&interface_choice("fastrouteiface", $rule->{'fastroute'}, 1),"\n";
print "$text{'edit_fastrouteip'}\n";
print &ui_textbox("fastrouteip", $rule->{'fastroute-ip'}, 13),
      " $text{'edit_opt'}</td> </tr>\n";

print "<tr> <td>",&ui_checkbox("reply_to", 1, $text{'edit_replyto'},
			       $rule->{'reply-to'}),"</td>\n";
print "<td>",&interface_choice("reply_toiface", $rule->{'reply-to'}, 1),"\n";
print "$text{'edit_fastrouteip'}\n";
print &ui_textbox("reply_toip", $rule->{'reply-to-ip'}, 13),
      " $text{'edit_opt'}</td> </tr>\n";

print "<tr> <td>",&ui_checkbox("keep", 1, $text{'edit_keep'},
			       $rule->{'keep'}),"</td>\n";
print "<td>",&ui_select("keepmode", $rule->{'keep'},
		[ [ "state", $text{'edit_keepstate'} ],
		  [ "frags", $text{'edit_keepfrags'} ] ]),"<td> </tr>\n";

print &ui_table_end();

if ($in{'new'}) {
	print &ui_form_end([ [ 'create', $text{'create'} ] ], "100%");
	}
else {
	print &ui_form_end([ [ 'save', $text{'save'} ],
			     [ 'delete', $text{'delete'} ] ], "100%");
	}
&ui_print_footer("", $text{'index_return'});

# yes_no_ignored_input(name)
sub yes_no_ignored_input
{
local $mode = $rule->{$_[0]} && $rule->{$_[0]."_not"} ? 2 :
	      $rule->{$_[0]} ? 1 : 0;
return &ui_radio($_[0], $mode,
		 [ [ 1, $text{'yes'} ],
		   [ 0, $text{'no'} ] ]);
}

# logging_options(prefix, split)
sub logging_options
{
local $pfx = $_[0];
local $rv;
local $ll = $rule->{$pfx."-level"};
local ($f, $p);
if ($ll =~ /^(\S+)\.(\S+)$/) {
	$p = $2; $f = $1;
	}
elsif ($ll =~ /\S/) {
	$p = $ll;
	}
$rv .= &ui_select($pfx."_pri", $p,
	  [ [ "", $text{'default'} ], map { [ $_ ] } @log_priorities ],
	  0, 0, 1);
$rv .= " $text{'edit_fac'} ";
$rv .= &ui_select($pfx."_fac", $f,
	  [ [ "", $text{'default'} ], map { [ $_ ] } @log_facilities ],
	  0, 0, 1)."\n";
$rv .= "<br>&nbsp;&nbsp;&nbsp;\n" if ($_[1]);
$rv .= &ui_checkbox($pfx."_body", 1, $text{'edit_log_body'},
		    $rule->{$pfx.'-body'})."\n";
$rv .= &ui_checkbox($pfx."_first", 1, $text{'edit_log_first'},
		    $rule->{$pfx.'-first'})."\n";
$rv .= &ui_checkbox($pfx."_orblock", 1, $text{'edit_log_orblock'},
		    $rule->{$pfx.'-or-block'})."\n";
return $rv;
}


